#!/usr/bin/perl -w
#
# Sensor for gathering vsftpd performance data for Zabbix
# Copyright (c) 2008 Hrvoje Shute
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# Changes and Modifications
# * Thu May  7 21:52:18 CEST 2009
#	- rewritten in perl
#       - rewritten sender to use only one connection

use strict;
use warnings;

# zabbix path
my $zabbix_sender="/usr/bin/zabbix_sender";
my $zabbix_confd="/etc/zabbix/zabbix_agentd.conf";
my $send_file="/var/tmp/zabbixSenderFileVsftpd";
my $zabbix_send_command="$zabbix_sender -c $zabbix_confd -i $send_file";
my $inputString = '';
my $hostname = `/bin/hostname -f`;
chomp($hostname);

# signal handling
$SIG{'ALRM'} = sub {
  local $SIG{TERM} = 'IGNORE';
  kill TERM => -$$;
  print "0\n";
};

local $SIG{TERM} = sub {
  local $SIG{TERM} = 'IGNORE';
  kill TERM => -$$;
  print "0\n";
};

alarm 25;

# vsftpd data
my $vsftplog="/var/log/vsftpd.log";
my $vsftpstate="/var/tmp/zabbix-vsftpd-offset";
my $logtail="/usr/sbin/logtail";
my $cmd = "$logtail -f$vsftplog -o$vsftpstate";
my $output = qx($cmd);

# get data
my %data;               #        print $output;
my @vsftpLines = split("\n",$output);
#my @tmpGrep = grep(/dovecot/, @tmpArr);
#$output = join("\n", @vsftpLines); print $output;
my @tmpArr = grep(/CONNECT/, @vsftpLines);
$data{'vsftpd.connections'} = @tmpArr;
@tmpArr = grep(/OK LOGIN/, @vsftpLines);
$data{'vsftpd.loginsuccess'} = @tmpArr;
@tmpArr = grep(/FAIL LOGIN/, @vsftpLines);
$data{'vsftpd.loginfailure'} = @tmpArr;
@tmpArr = grep(/OK UPLOAD/, @vsftpLines);
$data{'vsftpd.uploadsuccess'} = @tmpArr;
@tmpArr = grep(/FAIL UPLOAD/, @vsftpLines);
$data{'vsftpd.uploadfailure'} = @tmpArr;
@tmpArr = grep(/OK DOWNLOAD/, @vsftpLines);
$data{'vsftpd.downloadsuccess'} = @tmpArr;
@tmpArr = grep(/FAIL DOWNLOAD/, @vsftpLines);
$data{'vsftpd.downloadfailure'} = @tmpArr;
@tmpArr = grep(/OK DELETE/, @vsftpLines);
$data{'vsftpd.deletesuccess'} = @tmpArr;
@tmpArr = grep(/FAIL DELETE/, @vsftpLines);
$data{'vsftpd.deletefailure'} = @tmpArr;

#print "\n--$data{'dovecot.connectionsIMAPestablished'}!!";

# write data to variable
foreach my $key (keys %data){
  $inputString .= $hostname ." ". $key ." ". $data{$key} ."\n" if($data{$key}); #$server ." ". $hostname ." ". $port
}

# write everything to file
open FH, ">", $send_file or die("Can not open file $send_file!");
print FH $inputString;
close(FH);

# finally, send the data
if ( qx($zabbix_send_command) =~ /Failed(:)? 0/i ) {
   unlink ($send_file) or die("Can not remove file $send_file!");
   print("1\n");
   exit(0);
} else {
   unlink ($send_file) or die("Can not remove file $send_file!");
   print("0\n");
   exit(-1);
}
#EOF
